<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class LocaisAtendimentoFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /* INTEGRAÇÃO COM SM */
    public function getContratosConfigurarLocais()
    {

        $sql = "SELECT FCT_RELACESTRUTURACONT.*, FCT_CONTRATO.NRCONTRATOFCT 
                    FROM FCT_RELACESTRUTURACONT
                    JOIN FCT_CONTRATO
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCT_RELACESTRUTURACONT.NRSEQCONTRATO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getLocaisSM($parametros)
    {

        $sql = "SELECT DISTINCT FCT_RELACESTRUTURACONT.NRRELACESTRUTCONT, FCT_RELACESTRUTURACONT.NRESTRUTURAM, FCT_RELACESTRUTURACONT.NRSEQCONTRATO, ESTRUTURAM.NMESTRUTURAM
                    FROM FCT_RELACESTRUTURACONT
                    JOIN ESTRUTURAM
                    ON ESTRUTURAM.NRESTRUTURAM = FCT_RELACESTRUTURACONT.NRESTRUTURAM
                    WHERE FCT_RELACESTRUTURACONT.NRSEQCONTRATO IN ({$parametros})";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getClienteContrato($parametros)
    {

        $sql = "SELECT CLIENTE.CDCLIENTE 
                    FROM CLIENTE
                    JOIN FCT_CONTRATO
                      ON FCT_CONTRATO.NRPARCNEGOCIO = CLIENTE.NRPARCNEGOCIO
                    WHERE FCT_CONTRATO.NRSEQCONTRATO = $parametros";

        $retorno = $this->connection->executeQuery($sql)->fetchAllAssociative();
        return $retorno[0]['CDCLIENTE'];
    }

    public function verificarExistenciaLocais($parametro)
    {
        $sql = "SELECT NRLOCALCLIENTE, NRESTRUTURAM
                    FROM FCM_LOCALCLI
                    WHERE NRESTRUTURAM = $parametro
                    AND IDATIVO = 'S'";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getContratosRelacionadosLocais($parametros)
    {
        $sql = "SELECT FCM_RELACLOCALCONT.NRRELACLOCALCONT, FCM_RELACLOCALCONT.NRESTRUTURAM, FCM_RELACLOCALCONT.NRLOCALCLIENTE, FCM_RELACLOCALCONT.NRSEQCONTRATO, 
                    FCM_RELACLOCALCONT.NRRELACESTRUTCONT, FCT_CONTRATO.NRCONTRATOFCT
                    FROM FCM_RELACLOCALCONT
                    JOIN FCT_CONTRATO
                    ON FCT_CONTRATO.NRSEQCONTRATO = FCM_RELACLOCALCONT.NRSEQCONTRATO
                    WHERE NRESTRUTURAM = :NRESTRUTURAM";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function verificarExistenciaContratoRelacionado($parametros)
    {
        $sql = "SELECT COUNT(*) NRSEQCONTRATO
                    FROM FCM_RELACLOCALCONT
                    WHERE NRSEQCONTRATO = :NRSEQCONTRATO
                    AND NRESTRUTURAM = :NRESTRUTURAM
                    AND IDATIVO = 'S'";

        $st = $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $st[0]['NRSEQCONTRATO'];
    }

    public function inserirRelacaoLocalContrato($parametros)
    {
        $sql = "INSERT INTO FCM_RELACLOCALCONT (NRRELACLOCALCONT, NRESTRUTURAM, NRLOCALCLIENTE, NRSEQCONTRATO, NRRELACESTRUTCONT, IDATIVO, NRORG, DTINCLUSAO, 
                                                CDOPERINCLUSAO, NRORGINCLUSAO, DTULTATU, CDOPERULTATU, NRORGULTATU)
                VALUES (:NRRELACLOCALCONT, :NRESTRUTURAM, :NRLOCALCLIENTE, :NRSEQCONTRATO, :NRRELACESTRUTCONT, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO,
                        SYSDATE, :CDOPERULTATU, :NRORGULTATU)";

        $this->connection->executeStatement($sql, $parametros);
    }

    /* FIM INTEGRAÇÃO COM SM */
    public function getClientes()
    {

        $sql = "SELECT CDCLIENTE, NMFANTCLIE
                FROM CLIENTE
                where IDCLIEATIV = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getLocais($filter)
    {

        $sql = "SELECT LC.NRLOCALCLIENTE, LC.NMLOCALCLIENTE, TP.IDTIPOLOCALCLI, LC.NRTIPOLOCALCLI, LC.NRESTRUTURAM
                    FROM FCM_LOCALCLI LC
                    JOIN FCM_TIPOLOCALCLI TP
                    ON TP.NRTIPOLOCALCLI = LC.NRTIPOLOCALCLI
                    WHERE TP.IDTIPOLOCALCLI <> 3 " . $filter;

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTiposDeLocais($parametros)
    {

        $sql = "SELECT NMTIPOLOCALCLI, NRTIPOLOCALCLI, IDTIPOLOCALCLI
                    FROM FCM_TIPOLOCALCLI
                   WHERE IDTIPOLOCALCLI IN (DECODE(:IDTIPOLOCALCLI, NULL, 1, DECODE(:IDTIPOLOCALCLI, 1, 2, DECODE (:IDTIPOLOCALCLI, 2, 3, NULL))))
       
                UNION ALL 
       
                SELECT NMTIPOLOCALCLI, NRTIPOLOCALCLI, IDTIPOLOCALCLI
                    FROM FCM_TIPOLOCALCLI
                   WHERE IDTIPOLOCALCLI IN (DECODE(:IDTIPOLOCALCLI, NULL, 1, DECODE(:IDTIPOLOCALCLI, 1, 2, DECODE (:IDTIPOLOCALCLI, 2, 2, NULL))))";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getChecklistPadrao()
    {

        $sql = "SELECT NMCHECKLIST, NRCHECKLIST
                  FROM FCM_CHECKLIST
                  WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getSetorLocalAtendimento()
    {

        $sql = "SELECT NRSETOR, NMSETOR
                    FROM FCM_SETOR
                    WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getClientesQueAgrupamLocais()
    {
        $sql = "SELECT DISTINCT C.CDCLIENTE, C.NMFANTCLIE, LC.DSCORIDLOCAL
            FROM CLIENTE C 
            JOIN FCM_LOCALCLI LC
            ON C.CDCLIENTE = LC.CDCLIENTE
            WHERE LC.IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getLocaisAgrupados($parametro)
    {
        $sql = "SELECT LOC.NMLOCALCLIENTE, LOC.NRLOCALCLIENTE, LOC.NRLOCALCLIENTEPAI, LOC.DSCODIGOLOCALCLI, LOC.CDCLIENTE,
                        TLOC.NMTIPOLOCALCLI, TLOC.IDTIPOLOCALCLI, LOC.NRTIPOLOCALCLI, LOC.NRCHECKLISTPADR, LOC.NRSETOR, 
                        LOC.DSCORIDLOCAL, LOC.NMABREVLOCALCLI, LOC.IDATIVO, LOC.NRESTRUTURAM
                FROM FCM_LOCALCLI LOC
                JOIN FCM_TIPOLOCALCLI TLOC
                ON TLOC.NRTIPOLOCALCLI = LOC.NRTIPOLOCALCLI
                WHERE LOC.CDCLIENTE = :CDCLIENTE
                AND LOC.IDATIVO = 'S'
                ORDER BY TLOC.IDTIPOLOCALCLI ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametro);
    }


    public function getCorCliente($parametros)
    {
        $sql = "SELECT DISTINCT DSCORIDLOCAL
                    FROM FCM_LOCALCLI
                    WHERE CDCLIENTE = :CDCLIENTE";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function atualizarLocal($parametros)
    {

        $sql = "UPDATE FCM_LOCALCLI
                    SET NMLOCALCLIENTE = :NMLOCALCLIENTE,
                        CDCLIENTE      = :CDCLIENTE,
                        NRTIPOLOCALCLI = :NRTIPOLOCALCLI,
                        NRLOCALCLIENTEPAI = :NRLOCALCLIENTEPAI,
                        NRCHECKLISTPADR = :NRCHECKLISTPADR,
                        IDATIVO = :IDATIVO,
                        DTULTATU = SYSDATE,
                        DSCODIGOLOCALCLI = :DSCODIGOLOCALCLI,
                        NMABREVLOCALCLI = :NMABREVLOCALCLI,
                        CDOPERULTATU = :CDOPERULTATU,
                        NRORGULTATU = :NRORGULTATU
                WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirNovoLocal($parametros)
    {

        $sql = "INSERT INTO FCM_LOCALCLI (NRLOCALCLIENTE, NMLOCALCLIENTE, CDCLIENTE ,NRTIPOLOCALCLI, NRLOCALCLIENTEPAI, NRCHECKLISTPADR, DSQRCODE, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DSCODIGOLOCALCLI, DSCORIDLOCAL, NMABREVLOCALCLI, NRESTRUTURAM)
                    VALUES ( :NRLOCALCLIENTE, 
                            :NMLOCALCLIENTE, 
                            :CDCLIENTE,
                            :NRTIPOLOCALCLI,
                            :NRLOCALCLIENTEPAI,
                            :NRCHECKLISTPADR,
                            :DSQRCODE,
                            :IDATIVO, 
                            :NRORG,
                            SYSDATE, 
                            :CDOPERINCLUSAO, 
                            :NRORGINCLUSAO, 
                            :DSCODIGOLOCALCLI,
                            :DSCORIDLOCAL,
                            :NMABREVLOCALCLI,
                            :NRESTRUTURAM)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function inserirRelacLocalChk($paramsChk)
    {
        $sql = "INSERT INTO FCM_RELACLOCALCHK (NRRELACLOCALCHK, NRLOCALCLIENTE, NRCHECKLIST, DTINCLUSAO, DTULTATU)
                    VALUES(:NRRELACLOCALCHK, :NRLOCALCLIENTE, :NRCHECKLIST, SYSDATE, SYSDATE)";

        $this->connection->executeStatement($sql, $paramsChk);
    }

    public function alterarCorCliente($parametros)
    {

        $sql = "UPDATE FCM_LOCALCLI
                    SET DSCORIDLOCAL = :DSCORIDLOCAL,
                        DTULTATU = SYSDATE, 
                        NRORGULTATU = :NRORGULTATU,
                        CDOPERULTATU = :CDOPERULTATU
                    WHERE CDCLIENTE = :CDCLIENTE";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function verificarExistenciaTarefas($parametros)
    {
        $sql = "SELECT NMCHECKLISTAGEND, DTHORAAGEND, NRCHECKLISTAGEND AS CODIGO
                FROM FCM_CHECKLISTAGEND 
                WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE
                AND IDATIVO = 'S'";
    
        return $this->connection->executeQuery($sql, ['NRLOCALCLIENTE' => $parametros])->fetchAllAssociative();
    }    


    public function verificarExistenciaHierarquia($parametros)
{
    $sql = "SELECT NMHIERARQUIAFUNC, NRHIERARQUIAFUNC AS CODIGO
            FROM FCM_HIERARQUIAFUNC 
            WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE
            AND IDATIVO = 'S'";

    return $this->connection->executeQuery($sql, ['NRLOCALCLIENTE' => $parametros])->fetchAllAssociative();
    }

    public function verificarSituacaoTarefa($parametros)
    {
        $nrLocalCliente = is_array($parametros) ? ($parametros['NRLOCALCLIENTE'] ?? null) : $parametros;
    
        if (!$nrLocalCliente) {
            throw new \InvalidArgumentException("O parâmetro 'NRLOCALCLIENTE' é obrigatório.");
        }
        $sql = "SELECT NMCHECKLISTAGEND, NRCHECKLISTAGEND, IDSITUACAOAGEND, DTHORAAGEND
                FROM FCM_CHECKLISTAGEND 
                WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE
                  AND IDATIVO = 'S'";
    
        $result = $this->connection->executeQuery($sql, ['NRLOCALCLIENTE' => $nrLocalCliente])->fetchAllAssociative();

        return $result;
    }

    public function verificarExistenciaFilhos($parametros)
    {
        $sql = "SELECT COUNT(*) FILHOS
                FROM FCM_LOCALCLI
                WHERE NRLOCALCLIENTEPAI = :NRLOCALCLIENTEPAI
                AND IDATIVO = 'S'";
    
        $st = $this->connection->executeQuery($sql, ['NRLOCALCLIENTEPAI' => $parametros])->fetchAllAssociative();
        return $st[0]['FILHOS'];
    }    
        
    public function verificaRelacionamentoLocalTarefas($parametros)
    {
        $sql = "SELECT COUNT(*) AS total FROM NRLOCALCLIENTEPAI = :NRLOCALCLIENTEPAI";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($parametros);
        return $stmt->fetch();
    }

    public function verificarTarefasRelacionadas($params)
    {
        if (!isset($params['NRLOCALCLIENTE'])) {
            throw new \InvalidArgumentException("Parâmetro 'NRLOCALCLIENTE' é obrigatório.");
        }

        $sql = "SELECT NMCHECKLISTAGEND 
            FROM FCM_CHECKLISTAGEND 
            WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE";

        $stmt = $this->connection->prepare($sql);
        $stmt->bindParam(':NRLOCALCLIENTE', $params['NRLOCALCLIENTE'], \PDO::PARAM_INT);
        $stmt->execute();

        return $stmt->fetchAll(\PDO::FETCH_ASSOC);
    }

    public function getTarefasAssociadas($params)
    {
        $sql = "SELECT COUNT(*) AS ITENS
                FROM FCM_CHECKLISTAGEND
                WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE
                  AND IDATIVO = 'S'";
    
        return $this->connection->executeQuery($sql, $params)->fetchOne();
    }

    public function verificarLocaisEmUso($params)
    {
        $sql = "SELECT FCM_CHECKLISTAGEND.NMCHECKLISTAGEND
            FROM FCM_CHECKLISTAGEND
            INNER JOIN FCM_LOCALCLI ON FCM_CHECKLISTAGEND.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTE
            WHERE FCM_CHECKLISTAGEND.NRLOCALCLIENTE = :NRLOCALCLIENTE
            AND FCM_CHECKLISTAGEND.IDATIVO = 'S'
        ";
    
        return $this->connection->executeQuery($sql, $params)->fetchOne();
    }

    public function inativarLocais($parametros)
{
                $sql = "UPDATE FCM_LOCALCLI
            SET IDATIVO = 'N',
                CDOPERULTATU = :CDOPERULTATU,
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU
            WHERE NRLOCALCLIENTE = :NRLOCALCLIENTE";

    $this->connection->executeStatement($sql, $parametros);
}

    // --- Funções da TAB DE TIPO DE LOCAL
    public function getTipoDeLocal()
    {
        $sql = "SELECT DISTINCT T1.NMTIPOLOCALCLI, T1.NRTIPOLOCALCLI, T1.IDTIPOLOCALCLI, T1.IDATIVO, T2.NRTIPOLOCALCLI as VALIDA
            FROM FCM_TIPOLOCALCLI T1
            LEFT JOIN FCM_LOCALCLI T2 ON T1.NRTIPOLOCALCLI = T2.NRTIPOLOCALCLI
            ORDER BY T1.IDTIPOLOCALCLI ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function validaTipoLocalUsado()
    {

        $sql = "SELECT NMTIPOLOCALCLI, NRTIPOLOCALCLI, IDTIPOLOCALCLI
                    FROM FCM_TIPOLOCALCLI 
                 ORDER BY IDTIPOLOCALCLI ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function excluirTipoLocal($parametros)
{
    if (is_array($parametros['NRTIPOLOCALCLI'])) {
        $parametros['NRTIPOLOCALCLI'] = implode(',', $parametros['NRTIPOLOCALCLI']);
    }

    $sql = "DELETE FROM FCM_TIPOLOCALCLI WHERE NRTIPOLOCALCLI IN (:NRTIPOLOCALCLI)";
    $this->connection->executeStatement($sql, $parametros);
}

public function verificaRelacionamentoLocais($parametros)
{
    $nrTipoLocalCli = is_array($parametros['NRTIPOLOCALCLI']) ? $parametros['NRTIPOLOCALCLI'][0] : $parametros['NRTIPOLOCALCLI'];

    $sql = "SELECT NMLOCALCLIENTE, NRLOCALCLIENTE, NRTIPOLOCALCLI
            FROM FCM_LOCALCLI
            WHERE NRTIPOLOCALCLI = :NRTIPOLOCALCLI";

    $retorno = $this->connection->executeQuery($sql, ['NRTIPOLOCALCLI' => $nrTipoLocalCli])->fetchAllAssociative();

    return $retorno;
}

public function verificaAtualizacaoLocais($parametros)
{
    if (empty($parametros['NRTIPOLOCALCLI'])) {
        return [];
    }

    $sql = "SELECT NMLOCALCLIENTE, NRLOCALCLIENTE, NRTIPOLOCALCLI
            FROM FCM_LOCALCLI
            WHERE NRTIPOLOCALCLI = :NRTIPOLOCALCLI
            AND IDATIVO = 'S'";

    $retorno = $this->connection->executeQuery($sql, ['NRTIPOLOCALCLI' => $parametros['NRTIPOLOCALCLI']])->fetchAllAssociative();

    return $retorno;
}

    public function atualizarTipoLocal($parametros)
    {

        $sql = "UPDATE FCM_TIPOLOCALCLI
                    SET NMTIPOLOCALCLI = :NMTIPOLOCALCLI,
                        IDTIPOLOCALCLI = :IDTIPOLOCALCLI,
                        IDATIVO = :IDATIVO,
                        NRORGULTATU    = :NRORGULTATU,
                        CDOPERULTATU   = :CDOPERULTATU,
                        DTULTATU = SYSDATE
                    WHERE NRTIPOLOCALCLI = :NRTIPOLOCALCLI";

        $this->connection->executeStatement($sql, $parametros);
    }

    

    public function inserirTipoLocal($parametros)
    {

        $sql = "INSERT INTO FCM_TIPOLOCALCLI (NRTIPOLOCALCLI, NMTIPOLOCALCLI, IDTIPOLOCALCLI, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, IDATIVO, DTINCLUSAO)
                    VALUES(:NRTIPOLOCALCLI, 
                           :NMTIPOLOCALCLI, 
                           :IDTIPOLOCALCLI,
                           :NRORG,
                           :NRORGINCLUSAO,
                           :CDOPERINCLUSAO, 
                           :IDATIVO,
                           SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    // ----------- FUNÇÕES GEOLOCALIZAÇÃO ------------
    public function getGeolocalizacao($params)
    {
        $sql = "SELECT NRLOCALCLIENTE, NRGEOLOCALIZACAO
                FROM FCM_LOCALCLI
                WHERE NRLOCALCLIENTE = $params";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function salvarGeolocalizacao($params, $filtrar)
    {
        $sql = "UPDATE FCM_LOCALCLI
                   SET NRGEOLOCALIZACAO = :NRGEOLOCALIZACAO,
                       NRORGULTATU = :NRORGULTATU,
                       CDOPERULTATU = :CDOPERULTATU " . $filtrar;

        $this->connection->executeStatement($sql, $params);
    }
    public function buscarNrAgrupamentoSimples($params)
    {
        $sql = "SELECT NRLOCALCLIENTE 
                FROM FCM_LOCALCLI
                WHERE NRLOCALCLIENTEPAI = $params";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
