<?php

namespace FM\Service;

use FM\Factory\HierarquiaUsuariosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;

class HierarquiaUsuariosService
{

    protected $HierarquiaUsuariosFactory;
    protected $Environment;
    protected $newCode;
    protected $FindService;

    public function __construct(EntityManager $entityManager, HierarquiaUsuariosFactory $HierarquiaUsuariosFactory, Environment $Environment, NewCode $newCode, FindService $FindService)
    {
        $this->connection                                       = $entityManager->getConnection();
        $this->HierarquiaUsuariosFactory                        = $HierarquiaUsuariosFactory;
        $this->Environment                                      = $Environment;
        $this->newCode                                          = $newCode;
        $this->FindService                                      = $FindService;
    }

    // Busca as hierarquias cadastradas
    public function getHierarquia()
    {
        return $this->HierarquiaUsuariosFactory->getHierarquia();
    }

    //Busca uma lista com os usuários cadastrados na hierarquia escolhida
    public function getHierarquiaDetalhes($parametros)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $parametros['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaDetalhes($param);
    }

    public function getHierarquiaFacilities($parametros)
    {
        $param = array(
            "NRFUNCIONARIO" => $parametros['NRFUNCIONARIO']
        );

        return $this->HierarquiaUsuariosFactory->getHierarquiaFacilities($param);
    }

    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarquiaUsuarios($params)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $params["find"]['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaUsuarios($param);
    }

    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarquiaPais($params)
    {
        $param = array(
            "NRHIERARQUIAFUNC" => $params["find"]['NRHIERARQUIAFUNC']
        );
        return $this->HierarquiaUsuariosFactory->getHierarquiaPais($param);
    }
    // Função para editar ou inserir nora hierarquia
    public function salvarHierarquia($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRHIERARQUIAFUNC"] != 0) {
                    $this->HierarquiaUsuariosFactory->editarHierarquia(array(
                        "NRHIERARQUIAFUNC"      => $row["NRHIERARQUIAFUNC"],
                        "IDATIVO"               => $row["IDATIVO"],
                        "NMHIERARQUIAFUNC"      => $row['NMHIERARQUIAFUNC'],
                        "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                        "NRORGULTATU"           => $nrorg,
                        "CDOPERULTATU"          => $operador
                    ));
                } else {
                    $this->HierarquiaUsuariosFactory->salvarHierarquia(array(
                        "NRHIERARQUIAFUNC"      => $this->newCode->getCodeByTableName("FCM_HIERARQUIAFUNC", $nrorg, 1, 10, true),
                        "IDATIVO"               => $row["IDATIVO"],
                        "NMHIERARQUIAFUNC"      => $row['NMHIERARQUIAFUNC'],
                        "NRLOCALCLIENTE"        => $row['NRLOCALCLIENTE'],
                        "NRORG"                 => $nrorg,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                        "NRORGULTATU"           => $nrorg,
                        "CDOPERULTATU"          => $operador
                    ));
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Insere um novo usuário à hierarquia
    public function inserirUsuarios($params)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $operadorI = $this->Environment->getCdOperador();

            foreach ($params['CDOPERADOR'] as $operador) {
                $nrfuncionarioshierarq =  $this->newCode->getCodeByTableName("FCM_FUNCIONARIOSHIERARQ", $nrorg, 1, 10, true);
                $paramsUsers = array(
                    'NRFUNCIONARIOSHIERARQ' => $nrfuncionarioshierarq,
                    "NRHIERARQUIAFUNC"      => $params['NRHIERARQUIAFUNC'],
                    'CDOPERADOR'            => $operador,
                    'CDOPERADORPAI'         => $params['CDOPERADORPAI'],
                    "NRORG"                 => $nrorg,
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $operadorI
                );
                $this->HierarquiaUsuariosFactory->inserirUsuarios($paramsUsers);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Deleta a hierarquia escolhida
    // Deleta a hierarquia escolhida
    public function excluirHierarquia($parametro)
    {
        $this->connection->beginTransaction();
    
        try {
            // Chama getFilhosAtivos passando o valor de NRS diretamente
            $filhosAtivos = $this->HierarquiaUsuariosFactory->getFilhosAtivos($parametro["NRS"]);
    
            // Verifica se há filhos ativos para a hierarquia a ser excluída
            if (isset($filhosAtivos['total']) && $filhosAtivos['total'] > 0) {
                throw new \Exception('Não é possível excluir a hierarquia. Existem filhos ativos.');
            }
    
            // Se não houver filhos ativos, prossegue com a exclusão
            $this->HierarquiaUsuariosFactory->excluirHierarquia($this->FindService->formatarArray($parametro["NRS"]));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    
    

// Função responsável por verificar se existem filhos ativos
private function verificaFilhosAtivos($nrs)
{
    $filhosAtivos = [];
    
    foreach ($nrs as $nr) {
        // Aqui você deve adaptar a consulta para verificar se há filhos ativos
        // Supondo que você tenha um método na fábrica que faz isso
        $resultados = $this->HierarquiaUsuariosFactory->getFilhosAtivos($nr);
        
        if (!empty($resultados)) {
            foreach ($resultados as $filho) {
                $filhosAtivos[] = $filho['CDOPERADOR']; // Ajuste isso conforme necessário
            }
        }
    }
    
    return $filhosAtivos;
}

    // Função responsável por retornar o array do PAI e de todos os filhos.
    public function selectFilhos($parametro)
    {
        $string = (implode(',', $parametro)); // transforma em string o array para concatenar com a query
        $retorno = $this->HierarquiaUsuariosFactory->buscarCdoperador($string); // Armazena na variavel retorno os valores dos filhos retornados

        if (empty($retorno)) { // verifica se o retorno não é vazio caso seja encerra o chamado da função devolvendo o parametro recebido
            return $parametro;
        }
        return $this->selectFilhos(array_merge($parametro, $this->getCdOeprador($retorno))); // faz uma chama recursiva da propria função passando como parâmetro
    }                                                                                        // os parametros recebidos junto com os filhos encontrados

    // Recebe o retorno do banco de dados e transforma em um array
    public function getCdOeprador($v)
    {
        $aux = []; // recebe os valores retornados
        foreach ($v as $cd) { // itera sobre os valores do BD
            $aux[] = $cd["CDOPERADOR"]; // aux recebe de cada elemento o valor indexo pela chave CDOPERADOR
        }
        return $aux; // retorna um novo array
    }

    // Deleta um ou mais usuários da hierarquia
    public function excluirUsuarios($parametro)
    {
        $this->connection->beginTransaction();

        $NRGERAL =  $this->selectFilhos($parametro["NRS"]); // armazena o NRGERAL o NR recebido e os filhos encontrados
        try {
            $param = implode(',', $NRGERAL);
            $this->HierarquiaUsuariosFactory->excluirUsuarios($param);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getLocalHierarquia()
    {
        return $this->HierarquiaUsuariosFactory->getLocalHierarquia();
    }

    public function getHierarquiaLocal()
    {
        return $this->HierarquiaUsuariosFactory->getHierarquiaLocal();
    }
}
