import {
    DialogService, LoadingService, Modal, ModalService, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import {
    Http, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';

export class Fm00002TipoLocalController {
    public modalErroExcluirTipoLocal!: Modal;

    public modalErroinativarTipoLocal!: Modal;

    public validaTipoLocal({ row }: IEventParam<TekGrid>) {
        if (row.VALIDA != null) {
            return false;
        }
        return true;
    }

    constructor() {
        this.inicializar();
    }

    private async inicializar() {
        await this.criarModais();
    }

    public async criarModais() {
        this.modalErroExcluirTipoLocal = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_0002_0002_modal_tipo_local', true);
        this.modalErroinativarTipoLocal = await ModalService.createFromJson('fm_00002_000_gerenciar_locais/modal/FM_0002_0002_modal_local_inativar', true);
 }

    public async abrirModalErro() {
        if (this.modalErroExcluirTipoLocal) {
            await this.modalErroExcluirTipoLocal.show();
        }
    }

    public fecharModalErro() {
        if (this.modalErroExcluirTipoLocal) {
            this.modalErroExcluirTipoLocal.hide();
        }
    }

    public async abrirModalErroInativar() {
        if (this.modalErroinativarTipoLocal) {
            await this.modalErroinativarTipoLocal.show();
        }
    }

    public fecharModalErroInativar() {
        if (this.modalErroinativarTipoLocal) {
            this.modalErroinativarTipoLocal.hide();
        }
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRTIPOLOCALCLI: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMTIPOLOCALCLI_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public atualizarTipoLocal({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            Http.post('/atualizarTipoLocal', {
                DRG: component.getEditedRows(),
            }).then((response) => {
                if (response.data.status === 'erro') {
                    this.montarModalErroInativar(response.data.relacionamento);
                }
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public async montarModalErroInativar(tarefas: any) {
        await this.abrirModalErroInativar();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-inativar-tipo-local');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public afterModify() {
        Metadata.getInstance<TekGrid>('tek-grid-tipo-local').datasource.get();
    }

    public excluirTipoLocal({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00002TipoLocalController.aposConfirmarDelete', 'Excluir');
    }

    public aposConfirmarDelete() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-tipo-local');
        DialogService.hide();
        LoadingService.show();

        Http.post('/excluirTipoLocal', {
            NRS: grid.selectedRows.map((element: any) => element.NRTIPOLOCALCLI),
        })
            .then((response) => {
                if (response.data) {
                    if (response.data.status === 'parcial') {
                        Utilities.showSuccessMessage();
                        this.montarModalErro(response.data.nao_excluidos);
                    } else if (response.data.status === 'ok') {
                        Utilities.showSuccessMessage();
                    }
                }
                grid.datasource.get();
            })
            .catch(() => {
                Utilities.showErrorMessage('Ocorreu um erro ao tentar excluir os tipos de locais.');
            })
            .finally(() => {
                grid.selectedRows = [];
                LoadingService.hide();
            });
    }

    public validacao({ row }: any) {
        return row.VALIDA !== null;
    }

    public async montarModalErro(naoExcluidos: any) {
        await this.abrirModalErro();

        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-tipo-local');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        naoExcluidos.forEach((element: any) => {
            const dados: any = {
                NMLOCALCLIENTE: element.NMLOCALCLIENTE,
                NRLOCALCLIENTE: element.NRLOCALCLIENTE,
                NRTIPOLOCALCLI: element.NRTIPOLOCALCLI,
            };
            grid.datasource.post(dados);
        });

        grid.datasource.get();
    }
}
